////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// audio.h
////////////////////////////////////////////////////////////////////////////////

#ifndef AUDIO_H
#define AUDIO_H

#include <map>
#include <string>
#include <vector>
#include "fmod.h"
#pragma comment (lib, "fmodvc.lib")

////////////////////////////////////////////////////////////////////////////////
// An interface that all audio files should support.
////////////////////////////////////////////////////////////////////////////////
class AudioFile
{
public:
	virtual ~AudioFile() { };
	const char * getFileName() { return mFileName.c_str(); };
	virtual void play() = 0;
	virtual void pause() = 0;
	virtual bool isPaused() = 0;
	virtual void resume() = 0;
	virtual void stop() = 0;
	virtual void setVolume(int volume) = 0;
protected:
	std::string mFileName;
};

////////////////////////////////////////////////////////////////////////////////
// Module supports sequenced music files like midi and mod.
////////////////////////////////////////////////////////////////////////////////
class Module : public AudioFile
{
public:
	Module(std::string & fileName);
	~Module();
	void play();
	void pause();
	bool isPaused();
	void resume();
	void stop();
	void setVolume(int volume);
private:
	FMUSIC_MODULE* mModule;
};

////////////////////////////////////////////////////////////////////////////////
// Sample supports sampled music like .mp3 and .wav files.
////////////////////////////////////////////////////////////////////////////////
class Sample : public AudioFile
{
public:
	Sample(std::string & fileName);
	~Sample();
	void play();
	void pause();
	bool isPaused();
	void resume();
	void stop();
	void setVolume(int volume);
private:
	int mChannel;
	FSOUND_SAMPLE* mSample;
};

////////////////////////////////////////////////////////////////////////////////
// AudioManager handles the details of loading various types of audio files.
// Only a title for the piece and the file path are needed.
////////////////////////////////////////////////////////////////////////////////
class AudioManager
{
public:
	~AudioManager();
	static AudioManager *Instance();
	void load(std::string title, std::string fileName);
	void unload(std::string title);
	void play(std::string title);
	void pause(std::string title);
	bool isPaused(std::string title);
	void resume(std::string title);
	void stop(std::string title);
	void setVolume(std::string title, int volume);
protected:
	AudioManager();
private:
	// singleton
	static AudioManager *mInstance;
	std::vector<std::string> mModuleExtensions;
	std::vector<std::string> mSampleExtensions;
	std::map<std::string, AudioFile *> mMusicList;
};

#endif
