////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// menu.cpp
////////////////////////////////////////////////////////////////////////////////

#include "menu.h"

////////////////////////////////////////////////////////////////////////////////
// Menu is a vertical menu which can be left-aligned or centered.
////////////////////////////////////////////////////////////////////////////////
void Menu::SetUp(const std::string &title, float left, float top, float spacing,
				 const Color &textColor, const Color &selectedColor, bool centered)
{
	mTitle = title;
	mLeft = left;
	mTop = top;
	mSpacing = spacing;
	mTextColor = textColor;
	mSelectedColor = selectedColor;
	mCentered = centered;
	mCurrentSelection = 0;
}

void Menu::AddSelection(const std::string &text)
{
	mSelections.push_back(text);
}

void Menu::SelectPrevious()
{
	mCurrentSelection--;
	if(mCurrentSelection < 0)
	{
		mCurrentSelection = mSelections.size() - 1;
	}
}

void Menu::SelectNext()
{
	mCurrentSelection++;
	if(mCurrentSelection >= mSelections.size())
	{
		mCurrentSelection = 0;
	}
}

int Menu::CurrentSelection() const
{
	return mCurrentSelection;
}

void Menu::Render() const
{
	glColor3f(mTextColor.getRed(), mTextColor.getGreen(), mTextColor.getBlue());
	GameFramework::Instance()->DrawText(mTitle, mLeft, mTop, mCentered);

	for(int i = 0; i < mSelections.size(); i++)
	{
		GameFramework::Instance()->DrawText(mSelections[i], mLeft,
								mTop - ((i + 1) * mSpacing) - 0.05, mCentered);
	}

	glColor3f(mSelectedColor.getRed(), mSelectedColor.getGreen(), mSelectedColor.getBlue());
	GameFramework::Instance()->DrawText(mSelections[mCurrentSelection], mLeft,
							mTop - ((mCurrentSelection + 1) * mSpacing) - 0.05, mCentered);
}

void Menu::CleanUp()
{
	mSelections.clear();
}

////////////////////////////////////////////////////////////////////////////////
// An Option is like a drop down menu without a visible drop down part.
// Only the currently selected option is displayed.
////////////////////////////////////////////////////////////////////////////////
void Option::SetUp(float left, float top, const Color &textColor, bool centered,
				   int initialSelection)
{
	mLeft = left;
	mTop = top;
	mTextColor = textColor;
	mCentered = centered;
	mCurrentOption = initialSelection;
}

void Option::AddOption(const std::string &text)
{
	mOptions.push_back(text);
}

void Option::PreviousOption()
{
	mCurrentOption--;
	if(mCurrentOption < 0)
	{
		mCurrentOption = 0;
	}
}

void Option::NextOption()
{
	mCurrentOption++;
	if(mCurrentOption >= mOptions.size())
	{
		mCurrentOption = mOptions.size() - 1;
	}
}

int Option::CurrentOption() const
{
	return mCurrentOption;
}

void Option::Render() const
{
	glColor3f(mTextColor.getRed(), mTextColor.getGreen(), mTextColor.getBlue());
	GameFramework::Instance()->DrawText(mOptions[mCurrentOption], mLeft, mTop, mCentered);
}

void Option::CleanUp()
{
	mOptions.clear();
}

int Option::getOptionCount() const
{
	return mOptions.size() - 1;
}
