////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// menu.h
////////////////////////////////////////////////////////////////////////////////

#ifndef MENU_H
#define MENU_H

#include "framework.h"

////////////////////////////////////////////////////////////////////////////////
// Menu is a vertical menu which can be left-aligned or centered.
////////////////////////////////////////////////////////////////////////////////
class Menu
{
public:
	void SetUp(const std::string &title, float left, float top, float spacing,
				const Color &textColor, const Color &selectedColor, bool centered = false);
	void AddSelection(const std::string &selection);
	void SelectNext();
	void SelectPrevious();
	int CurrentSelection() const;
	void Render() const;
	void CleanUp();
private:
	std::string mTitle;
	std::vector<std::string> mSelections;
	int mCurrentSelection;
	float mLeft;
	float mTop;
	float mSpacing;
	Color mTextColor;
	Color mSelectedColor;
	bool mCentered;
};

////////////////////////////////////////////////////////////////////////////////
// An Option is like a drop down menu without a visible drop down part.
// Only the currently selected option is displayed.
////////////////////////////////////////////////////////////////////////////////
class Option
{
public:
	void SetUp(float left, float top, const Color &textColor,
				bool centered = false, int initialSelection = 0);
	void AddOption(const std::string &text);
	void NextOption();
	void PreviousOption();
	int CurrentOption() const;
	int getOptionCount() const;
	void Render() const;
	void CleanUp();
private:
	std::vector<std::string> mOptions;
	int mCurrentOption;
	float mLeft;
	float mTop;
	Color mTextColor;
	bool mCentered;
};

#endif
