////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// particle.h
////////////////////////////////////////////////////////////////////////////////

#ifndef PARTICLE_H
#define PARTICLE_H

#include "framework.h"

////////////////////////////////////////////////////////////////////////////////
// A Particle is an individual component of a particle system having position,
// velocity, and color.
////////////////////////////////////////////////////////////////////////////////
class Particle
{
public:
	void setColor(float red, float green, float blue);
	void setPosition(double x, double y, double z);
	void setVelocity(double x, double y, double z);
	void setLifespan(double lifespan);
	bool shouldDie();
	void update(double elapsedTime);
	void render();
private:
	Color mColor;
	Vector3d mPosition;
	Vector3d mVelocity;
	Event mDeathClock;
};

////////////////////////////////////////////////////////////////////////////////
// A ParticleSystem manages a collection of particles.
////////////////////////////////////////////////////////////////////////////////
class ParticleSystem
{
public:
	~ParticleSystem();
	void setSpawnCount(int count);
	void setLifetimeRange(double min, double max);
	void setVelocityMagnitudeRange(double min, double max);
	void setColorRange(double minRed, double maxRed, double minGreen,
						double maxGreen, double minBlue, double maxBlue);
	void setGenerationLimit(double delay);	// set minimum time between generation
	virtual void generate() = 0;			// particles are generated in different ways
											// depending on the desired effect
	void update(double elapsedTime);		// simulate the physics
	void render();

protected:
	int mSpawnCount;		// number of particles to create per generate() call
	double mMinLifetime;
	double mMaxLifetime;
	double mMinMagnitude;
	double mMaxMagnitude;
	double mGenerationDelay;// minimum time in seconds between generation
	Event mGenerationLimit;	// controls frequency of particle generation
	Color mMinColor;
	Color mMaxColor;
	std::vector<Particle> mParticles;
};

class Exhaust : public ParticleSystem
{
public:
	void setOriginReference(Vector3d* origin);
	void setVelocityReference(Vector3d* velocity);
	void setAngleRange(double angle);	// maximum angle of deviation from mVelocity
	void setReferenceAngle(double *angle);
	void generate();
private:
	Vector3d *mOrigin;		// origin of particle generation
	Vector3d *mVelocity;	// initial direction of particle movement
	double mAngle;			// controls spread of particles generated
	double *mReferenceAngle;// 
};

#endif
