////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// timer.cpp
////////////////////////////////////////////////////////////////////////////////

#include "timer.h"

////////////////////////////////////////////////////////////////////////////////
// Timer provides the ability to get elapsed time between frames.
////////////////////////////////////////////////////////////////////////////////

Timer::Timer()
{
	QueryPerformanceFrequency(&mFrequency);
	Update();
}

// Updates the timer so the frames per second can be calculated.
void Timer::Update()
{
	LARGE_INTEGER tempTimer;

	QueryPerformanceCounter(&tempTimer);
	mDelta.QuadPart = tempTimer.QuadPart - mLastQuery.QuadPart;
	mLastQuery.QuadPart = tempTimer.QuadPart;
}

// Returns the elapsed time in milliseconds since the function was last called.
double Timer::GetDelta() const
{
	return (double)(mDelta.QuadPart) / (double)(mFrequency.QuadPart);
}

////////////////////////////////////////////////////////////////////////////////
// Events make setting time delays and checking when they have expired easier.
////////////////////////////////////////////////////////////////////////////////
// Constructor sets default delay of 1.0 second.
Event::Event()
{
	mTimeElapsed = 0.0;
	mTimeToDelay = 1.0;
}

// Set a timer to signal an event after the specified amount of time has elapsed.
void Event::SetDelay(double seconds)
{
	mTimeElapsed = 0.0;
	mTimeToDelay = seconds;
}

// Tell the event how much time has passed since last call.
void Event::Update(double timeDelta)
{
	mTimeElapsed += timeDelta;
}

// Returns true if the delay time has passed.
bool Event::HasSignaled() const
{
	return mTimeElapsed >= mTimeToDelay;
}
