////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// types.h
////////////////////////////////////////////////////////////////////////////////

#ifndef TYPES_H
#define TYPES_H

#include <stdlib.h>
#include <assert.h>
#include <math.h>

const static int VECTOR_DIMENSIONS = 3;
static const int X = 0;
const static int Y = 1;
const static int Z = 2;

const static int COLOR_COMPONENTS = 3;
static const int RED = 0;
const static int GREEN = 1;
const static int BLUE = 2;

const double PI = 3.141592653589793238462;

////////////////////////////////////////////////////////////////////////////////
// A Color stores red, green, and blue components of a color and is simple
// enough to be entirely defined in the header file.
////////////////////////////////////////////////////////////////////////////////
class Color
{
public:
	// constructors
	Color() { setRed(0.0); setGreen(0.0); setBlue(0.0); };
	Color(float r, float g, float b) { setRed(r); setGreen(g); setBlue(b); };
	// accessors
	float getRed() const { return color[RED]; };
	float getGreen() const { return color[GREEN]; };
	float getBlue() const { return color[BLUE]; };
	// mutators
	void setRed(float r) { assert(0.0 <= r && r <= 1.0); color[RED] = r; };
	void setGreen(float g) { assert(0.0 <= g && g <= 1.0); color[GREEN] = g; };
	void setBlue(float b) { assert(0.0 <= b && b <= 1.0); color[BLUE] = b; };
private:
	float color[COLOR_COMPONENTS];
};

////////////////////////////////////////////////////////////////////////////////
// A Vector3d is a three dimensional vector.
////////////////////////////////////////////////////////////////////////////////
class Vector3d
{
public:
	// constructors
	Vector3d() { setX(0.0); setY(0.0); setZ(0.0); };
	Vector3d(double x, double y, double z) { setX(x); setY(y); setZ(z); };
	// algebraic operations
	Vector3d operator + (const Vector3d &other) const;
	Vector3d operator - (const Vector3d &other) const;
	Vector3d operator * (double scalar) const;
	Vector3d operator / (double scalar) const;
	// vector operations
	double magnitude() const;
	void normalize() { *this = *this / magnitude(); };
	double dotProduct(const Vector3d &other) const;
	Vector3d crossProduct(const Vector3d &other) const;
	Vector3d rotateInXY(double mAngle) const;
	// accessors
	const double *getVector();
	double getX() const { return vector[X]; };
	double getY() const { return vector[Y]; };
	double getZ() const { return vector[Z]; };
	// mutators
	void setX(double x) { vector[X] = x; };
	void setY(double y) { vector[Y] = y; };
	void setZ(double z) { vector[Z] = z; };
private:
	double vector[VECTOR_DIMENSIONS];
};
// scalar multiplied in front of Vector must be declared outside class
Vector3d operator * (double scalar, Vector3d other);

// treat the 3d vectors as 2d by ignoring the z component
double Determinant2d(const Vector3d &a, const Vector3d &b);

// returns true when the line segment with endpoints a1 and a2 
// intersects the line segment with endpoints b1 and b2
bool Intersect(const Vector3d &a1, const Vector3d &a2, const Vector3d &b1, const Vector3d &b2);

// returns a double randomly chosen within the range of the min and max
double getInRange(double min, double max);

#endif
