////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// gsCount.cpp
////////////////////////////////////////////////////////////////////////////////

#include "gameStates.h"

////////////////////////////////////////////////////////////////////////////////
// CountState displays a count down until the gameplay begins.
////////////////////////////////////////////////////////////////////////////////

// Declare the single instance.
CountState *CountState::mInstance = 0;

// Set up timer events for countdown to gameplay.
void CountState::Initialize()
{
	mSecondsElapsed = 0;
	mSecond.SetDelay(1.0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0.0, 1.0, 0.0, 1.0);
}

// Ignore key presses.
void CountState::HandleInput(unsigned char key)
{
	if(key)
	{
		GameFramework::Instance()->SetState(PilotState::Instance());
	}
}

// Draw the countdown depending on what events have happened.
void CountState::Render()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glColor3f(1.0, 1.0, 1.0);
	GameFramework::Instance()->DrawFPS();

	glColor3f(0.0, 0.8, 0.0);
	switch(mSecondsElapsed)
	{
	case 0:
		GameFramework::Instance()->DrawText("3", 0.0, 0.5, true);
		break;
	case 1:
		GameFramework::Instance()->DrawText("2", 0.0, 0.5, true);
		break;
	case 2:
		GameFramework::Instance()->DrawText("1", 0.0, 0.5, true);
		break;
	case 3:
		GameFramework::Instance()->DrawText("Go!", 0.0, 0.5, true);
		break;
	default:
		GameFramework::Instance()->SetState(PilotState::Instance());
		break;
	}
}

// Update the counters for the three events.
void CountState::Update(double timeDelta, unsigned char *keyBuffer)
{
	mSecond.Update(timeDelta);
	if(mSecond.HasSignaled())
	{
		++mSecondsElapsed;
		mSecond.SetDelay(1.0);
	}
}

void CountState::CleanUp()
{
}
