////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// gsMenu.cpp
////////////////////////////////////////////////////////////////////////////////

#include "gameStates.h"

////////////////////////////////////////////////////////////////////////////////
// CreditsState displays the credits among swirling stars.
////////////////////////////////////////////////////////////////////////////////

// Declare the single instance.
CreditsState *CreditsState::mInstance = 0;

// Load and play music.
void CreditsState::Initialize()
{
	// make background black and enable alpha blending
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	// allocate memory for stars, remember to delete
	mPoints = new Vector3d[STAR_COUNT_CREDITS];
	for(int i = 0; i < STAR_COUNT_CREDITS; i++)
	{
		mPoints[i].setX((float)rand() / RAND_MAX);
		mPoints[i].setY((float)rand() / RAND_MAX);
		mPoints[i].setZ((float)rand() / RAND_MAX);
	}

	AudioManager::Instance()->load("Mission", "assets\\audio\\mission.mid");
	AudioManager::Instance()->setVolume("Mission", 100);
	AudioManager::Instance()->play("Mission");

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0, (float)glutGet(GLUT_WINDOW_WIDTH)/glutGet(GLUT_WINDOW_HEIGHT),
					0.1, 100.0);

	mX = 0.0;
	mY = 0.0;
	mZ = 0.0;
	mFade = 0.0;
	mCurrentCredit = 0;
	mFadingIn = true;
	mFadingOut = false;
	mFadeOut.SetDelay(BEGIN_FADE);
	mNextCredit.SetDelay(TRANSITION_FADE);
}

// Any key press goes to the main menu.
void CreditsState::HandleInput(unsigned char key)
{
	if(key)
	{
		GameFramework::Instance()->SetState(MenuState::Instance());
	}
}

// draw the star field and credits
void CreditsState::Render()
{
	glClear(GL_COLOR_BUFFER_BIT| GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(0.2 * sin(mX) + 0.4, 0.2 * sin(mY) + 0.4, 0.2 * sin(mZ) + 0.4,
				0.5, 0.5, 0.5, 0.0, 1.0, 0.0);

	mX += 0.05 * mTimeDelta;
	mY += 0.1 * mTimeDelta;
	mZ += 0.15 * mTimeDelta;

	if(mX >= 2.0 * PI)
		mX = 0.0;
	if(mY >= 2.0 * PI)
		mY = 0.0;
	if(mZ >= 2.0 * PI)
		mZ = 0.0;

	// white stars (and FPS)
	glColor3f(1.0, 1.0, 1.0);
	glBegin(GL_POINTS);
		for(int i = 0; i < STAR_COUNT_CREDITS; i++)
		{
			glVertex3f(mPoints[i].getX(), mPoints[i].getY(), mPoints[i].getZ());
		}
	glEnd();
	GameFramework::Instance()->DrawFPS();

	// don't fade out from 'The End', but wait for keypress
	if((6 != mCurrentCredit) || (0.99 > mFade))
	{
		// update the alpha for credits
		if(mFadingIn)
		{
			mFade += 0.33 * mTimeDelta;
		}
		if(mFadingOut)
		{
			mFade -= 0.33 * mTimeDelta;
		}

		// clamp the value of fade to the range 0.0 to 1.0
		if(mFade > 1.0)
		{
			mFade = 1.0;
		}
		if(mFade < 0.0)
		{
			mFade = 0.0;
		}

		// reset timers for each credit
		if(mFadeOut.HasSignaled())
		{
			mFadingIn = false;
			mFadingOut = true;
			mFadeOut.SetDelay(TRANSITION_FADE);
			if(4 == mCurrentCredit)
			{
				mFadeOut.SetDelay(TRANSITION_FADE + 6.0);
			}
		}
		if(mNextCredit.HasSignaled())
		{
			mFadingIn = true;
			mFadingOut = false;
			mNextCredit.SetDelay(TRANSITION_FADE);
			if(4 == mCurrentCredit)
			{
				mNextCredit.SetDelay(TRANSITION_FADE + 6.0);
			}
			mCurrentCredit++;
		}

		glColor4f(1.0, 1.0, 1.0, mFade);
	}

	switch(mCurrentCredit)
	{
	case 0:
		GameFramework::Instance()->DrawText("Programming", 0.0, 0.54, true);
		GameFramework::Instance()->DrawText("Jace Miller", 0.0, 0.46, true);
		break;
	case 1:
		GameFramework::Instance()->DrawText("Music", 0.0, 0.7, true);
		GameFramework::Instance()->DrawText("\"Space Mission\"", 0.0, 0.56, true);
		GameFramework::Instance()->DrawText("Jace Miller", 0.0, 0.5, true);
		GameFramework::Instance()->DrawText("\"New 1\"", 0.0, 0.36, true);
		GameFramework::Instance()->DrawText("Anonymous", 0.0, 0.3, true);
		break;
	case 2:
		GameFramework::Instance()->DrawText("More Music", 0.0, 0.7, true);
		GameFramework::Instance()->DrawText("\"The Jedi\"", 0.0, 0.56, true);
		GameFramework::Instance()->DrawText("Star Wars: Knights of the Old Republic",
											0.0, 0.5, true);
		GameFramework::Instance()->DrawText("\"Kharak System\"", 0.0, 0.36, true);
		GameFramework::Instance()->DrawText("Homeworld", 0.0, 0.3, true);
		break;
	case 3:
		GameFramework::Instance()->DrawText("Compelling Story", 0.0, 0.54, true);
		GameFramework::Instance()->DrawText("Jace Miller", 0.0, 0.46, true);
		break;
	case 4:
		GameFramework::Instance()->DrawText("Inspiration", 0.0, 0.54, true);
		GameFramework::Instance()->DrawText("Space Lander", 0.0, 0.46, true);
		break;
	case 5:
		GameFramework::Instance()->DrawText("\"He determines the number of the stars",
											0.0, 0.66, true);
		GameFramework::Instance()->DrawText("and calls them each by name.",
											0.0, 0.58, true);
		GameFramework::Instance()->DrawText("Great is our Lord and mighty in power;",
											0.0, 0.5, true);
		GameFramework::Instance()->DrawText("his understanding has no limit.\"",
											0.0, 0.42, true);
		GameFramework::Instance()->DrawText("- Psalm 147: 4, 5", 0.0, 0.32, true);
		break;
	case 6:
		GameFramework::Instance()->DrawText("The End", 0.0, 0.52, true);
		break;
	}
}

// update timers
void CreditsState::Update(double timeDelta, unsigned char *keyBuffer)
{
	mTimeDelta = timeDelta;
	mNextCredit.Update(timeDelta);
	mFadeOut.Update(timeDelta);
}

// Free the loaded music.
void CreditsState::CleanUp()
{
	AudioManager::Instance()->unload("Mission");
	delete [] mPoints;
}
