////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// gsMenu.cpp
////////////////////////////////////////////////////////////////////////////////

#include "gameStates.h"

////////////////////////////////////////////////////////////////////////////////
// MenuState displays the main menu.
////////////////////////////////////////////////////////////////////////////////

// Declare the singleton instance.
MenuState *MenuState::mInstance = 0;

// Prepare the main menu's options and begin music.
void MenuState::Initialize()
{
	// black background
	glClearColor(0.0, 0.0, 0.0, 1.0);

	// create menu choices
	mMainMenu.SetUp("Main Menu", 0.0, 0.75, 0.1, Color(1.0, 1.0, 1.0),
					Color(0.0, 1.0, 0.0), true);
	mMainMenu.AddSelection("Play Game");
	mMainMenu.AddSelection("Options");
	mMainMenu.AddSelection("Credits");
	mMainMenu.AddSelection("Exit");

	// load and play music
	AudioManager::Instance()->load("Homeworld", "assets\\audio\\Homeworld.mp3");
	AudioManager::Instance()->setVolume("Homeworld", 255);
	if(AudioManager::Instance()->isPaused("Homeworld"))
	{
		AudioManager::Instance()->resume("Homeworld");
	}
	else
	{
		AudioManager::Instance()->play("Homeworld");
	}

	mSelectorHeight = 0.0;
	mSelectorHeight = mMainMenu.CurrentSelection() * -0.1 + 0.61;

	GameFramework::Instance()->SetSharedData(MissionLevel, 0);
}

// Handle main menu responses to keyboard input.
void MenuState::HandleInput(unsigned char key)
{
	// ESCAPE exits program
	if(27 == key)
	{
		GameFramework::Instance()->Shutdown();
	}

	// Crude level select for demoing levels
	if('3' == key)
	{
		GameFramework::Instance()->SetSharedData(MissionLevel, 3);
	}
	if('2' == key)
	{
		GameFramework::Instance()->SetSharedData(MissionLevel, 2);
	}
	if('1' == key)
	{
		GameFramework::Instance()->SetSharedData(MissionLevel, 1);
	}
	if('0' == key)
	{
		GameFramework::Instance()->SetSharedData(MissionLevel, 0);
	}

	// 'A' key returns to title screen
	if('a' == key)
	{
		AudioManager::Instance()->unload("Homeworld");
		GameFramework::Instance()->SetState(TitleState::Instance());
	}

	// arrow keys change selction
	if(GLUT_KEY_UP == key)
	{
		mMainMenu.SelectPrevious();
		mSelectorHeight = mMainMenu.CurrentSelection() * -0.1 + 0.61;
	}
	if(GLUT_KEY_DOWN == key)
	{
		mMainMenu.SelectNext();
		mSelectorHeight = mMainMenu.CurrentSelection() * -0.1 + 0.61;
	}

	// ENTER or SPACE selects a menu option
	if((13 == key) || (' ' == key))
	{
		switch(mMainMenu.CurrentSelection())
		{
		case(0):
			AudioManager::Instance()->unload("Homeworld");
			// initialize ship information with each new game
			Ship::Instance()->Initialize();
			GameFramework::Instance()->SetState(StoryState::Instance());
			break;
		case(1):
			AudioManager::Instance()->pause("Homeworld");
			GameFramework::Instance()->SetState(OptionState::Instance());
			break;
		case(2):
			AudioManager::Instance()->unload("Homeworld");
			GameFramework::Instance()->SetState(CreditsState::Instance());
			break;
		case(3):
			AudioManager::Instance()->unload("Homeworld");
			GameFramework::Instance()->Shutdown();
			break;
		}
	}
}

// draw the menu options
void MenuState::Render()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// do this for every state that has a spaceship pointer (mainMenu, options, pause)
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0.0, 1.0, 0.0, 1.0);

	glColor3f(0.0, 1.0, 0.0);
	glBegin(GL_LINE_LOOP);
		glVertex3f(0.36, mSelectorHeight + 0.03, 0.0);
		glVertex3f(0.36, mSelectorHeight - 0.03, 0.0);
		glVertex3f(0.42, mSelectorHeight, 0.0);
	glEnd();

	glColor3f(1.0, 1.0, 1.0);
	GameFramework::Instance()->DrawFPS();

	mMainMenu.Render();
}

// nothing changes here that is not covered in HandleInput()
void MenuState::Update(double timeDelta, unsigned char *keyBuffer)
{
}

// Remove the menu
void MenuState::CleanUp()
{
	mMainMenu.CleanUp();
}
