////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// gsOption.cpp
////////////////////////////////////////////////////////////////////////////////

#include "gameStates.h"

////////////////////////////////////////////////////////////////////////////////
// CreditsState displays the credits among swirling stars.
////////////////////////////////////////////////////////////////////////////////

// Declare the single instance.
OptionState *OptionState::mInstance = 0;

// Set up menu with options.
void OptionState::Initialize()
{
	int fullscreen = 0;
	int displayFPS = 0;
	int difficulty = 0;
	mJustChangedFullscreen = false;

	// black background
	glClearColor(0.0, 0.0, 0.0, 1.0);

	// create menu choices
	mOptionMenu.SetUp("Options Menu", 0.0, 0.75, 0.1, Color(1.0, 1.0, 1.0),
						Color(0.0, 0.3, 1.0), true);
	mOptionMenu.AddSelection("Fullscreen");
	mOptionMenu.AddSelection("Display FPS");
	mOptionMenu.AddSelection("Difficulty");
	//mOptionMenu.AddSelection("Apply Settings");
	mOptionMenu.AddSelection("Main Menu");

	// set options to settings loaded from file
	fullscreen = GameFramework::Instance()->GetSharedData(Fullscreen);
	displayFPS = GameFramework::Instance()->GetSharedData(DisplayFPS);
	difficulty = GameFramework::Instance()->GetSharedData(Difficulty);

	mFullscreen.SetUp(0.7, 0.6, Color(0.0, 0.3, 1.0), false, fullscreen);
	mFullscreen.AddOption("Windowed");
	mFullscreen.AddOption("Fullscreen");

	mDisplayFPS.SetUp(0.7, 0.5, Color(0.0, 0.3, 1.0), false, displayFPS);
	mDisplayFPS.AddOption("Hide");
	mDisplayFPS.AddOption("Show");

	mDifficulty.SetUp(0.7, 0.4, Color(0.0, 0.3, 1.0), false, difficulty);
	mDifficulty.AddOption("Easy");
	mDifficulty.AddOption("Medium");
	mDifficulty.AddOption("Hard");

	mSelectorHeight = 0.0;
	mSelectorHeight = mOptionMenu.CurrentSelection() * -0.1 + 0.61;

	AudioManager::Instance()->resume("Homeworld");
}

// Handle option menu responses to keyboard input.
void OptionState::HandleInput(unsigned char key)
{
	// ESCAPE exits to main menu without making changes
	if(27 == key)
	{
		AudioManager::Instance()->pause("Homeworld");
		GameFramework::Instance()->SetState(MenuState::Instance());
	}

	// arrow keys change selection and options within selection
	if(GLUT_KEY_UP == key)
	{
		mOptionMenu.SelectPrevious();
		mSelectorHeight = mOptionMenu.CurrentSelection() * -0.1 + 0.61;
	}
	if(GLUT_KEY_DOWN == key)
	{
		mOptionMenu.SelectNext();
		mSelectorHeight = mOptionMenu.CurrentSelection() * -0.1 + 0.61;
	}
	if(GLUT_KEY_LEFT == key)
	{
		switch(mOptionMenu.CurrentSelection())
		{
		case(0):
			mFullscreen.PreviousOption();
			GameFramework::Instance()->SetSharedData(Fullscreen, mFullscreen.CurrentOption());
			mJustChangedFullscreen = true;
			break;
		case(1):
			mDisplayFPS.PreviousOption();
			GameFramework::Instance()->SetSharedData(DisplayFPS, mDisplayFPS.CurrentOption());
			break;
		case(2):
			mDifficulty.PreviousOption();
			GameFramework::Instance()->SetSharedData(Difficulty, mDifficulty.CurrentOption());
			break;
		}
	}
	if(GLUT_KEY_RIGHT == key)
	{
		switch(mOptionMenu.CurrentSelection())
		{
		case(0):
			mFullscreen.NextOption();
			GameFramework::Instance()->SetSharedData(Fullscreen, mFullscreen.CurrentOption());
			mJustChangedFullscreen = true;
			break;
		case(1):
			mDisplayFPS.NextOption();
			GameFramework::Instance()->SetSharedData(DisplayFPS, mDisplayFPS.CurrentOption());
			break;
		case(2):
			mDifficulty.NextOption();
			GameFramework::Instance()->SetSharedData(Difficulty, mDifficulty.CurrentOption());
			break;
		}
	}

	// ENTER or SPACE selects a menu option
	if((13 == key) || (' ' == key))
	{
		// return to main menu
		if(3 == mOptionMenu.CurrentSelection())
		{
			AudioManager::Instance()->pause("Homeworld");
			GameFramework::Instance()->SetState(MenuState::Instance());
		}
	}
}

// Draw the ship pointer, FPS (if visible), and menu with options
void OptionState::Render()
{
	bool drawRight = true;
	bool drawLeft = true;

	glClear(GL_COLOR_BUFFER_BIT| GL_DEPTH_BUFFER_BIT);

	// do this for every state that has a spaceship pointer (mainMenu, options, pause)
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0.0, 1.0, 0.0, 1.0);

	glColor3f(0.0, 0.3, 1.0);
	glBegin(GL_LINE_LOOP);
		glVertex3f(0.34, mSelectorHeight + 0.03, 0.0);
		glVertex3f(0.34, mSelectorHeight - 0.03, 0.0);
		glVertex3f(0.40, mSelectorHeight, 0.0);
	glEnd();

	// decide which arrows to draw based on current option and selection
	if(mOptionMenu.CurrentSelection() < 3) // first 3 selections have options
	{
		switch(mOptionMenu.CurrentSelection())
		{
		case(0):
			if(mFullscreen.CurrentOption() >= mFullscreen.getOptionCount())
				drawRight = false;
			else
				drawRight = true;
			if(0 >= mFullscreen.CurrentOption())
				drawLeft = false;
			else
				drawLeft = true;
			break;
		case(1):
			if(mDisplayFPS.CurrentOption() >= mDisplayFPS.getOptionCount())
				drawRight = false;
			else
				drawRight = true;
			if(0 >= mDisplayFPS.CurrentOption())
				drawLeft = false;
			else
				drawLeft = true;
			break;
		case(2):
			if(mDifficulty.CurrentOption() >= mDifficulty.getOptionCount())
				drawRight = false;
			else
				drawRight = true;
			if(0 >= mDifficulty.CurrentOption())
				drawLeft = false;
			else
				drawLeft = true;
			break;
		}
		if(drawRight)
		{
			glBegin(GL_LINE_LOOP);
				glVertex3f(0.84, mSelectorHeight + 0.015, 0.0);
				glVertex3f(0.84, mSelectorHeight - 0.015, 0.0);
				glVertex3f(0.87, mSelectorHeight, 0.0);
			glEnd();
		}
		if(drawLeft)
		{
			glBegin(GL_LINE_LOOP);
				glVertex3f(0.69, mSelectorHeight + 0.015, 0.0);
				glVertex3f(0.69, mSelectorHeight - 0.015, 0.0);
				glVertex3f(0.66, mSelectorHeight, 0.0);
			glEnd();
		}
	}

	glColor3f(1.0, 1.0, 1.0);
	GameFramework::Instance()->DrawFPS();

	mOptionMenu.Render();
	mFullscreen.Render();
	mDisplayFPS.Render();
	mDifficulty.Render();
}

// nothing changes here that is not covered in HandleInput()
void OptionState::Update(double timeDelta, unsigned char *keyBuffer)
{
	if(mJustChangedFullscreen)
		GameFramework::Instance()->UpdateFullscreenStatus();
	mJustChangedFullscreen = false;
}

// Remove the menu
void OptionState::CleanUp()
{
	mOptionMenu.CleanUp();
	mFullscreen.CleanUp();
	mDisplayFPS.CleanUp();
	mDifficulty.CleanUp();
}
