////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// gsPause.cpp
////////////////////////////////////////////////////////////////////////////////

#include "gameStates.h"

////////////////////////////////////////////////////////////////////////////////
// Pause State allows you to continue or quit the current mission.
////////////////////////////////////////////////////////////////////////////////

// Declare the singleton instance.
PauseState *PauseState::mInstance = 0;

// Return to game or main menu.
void PauseState::Initialize()
{
	// black background
	glClearColor(0.0, 0.0, 0.0, 1.0);

	// create menu choices
	mPauseMenu.SetUp("Pause Menu", 0.0, 0.69, 0.1, Color(1.0, 1.0, 1.0),
						Color(1.0, 0.2, 0.0), true);
	mPauseMenu.AddSelection("Continue");
	mPauseMenu.AddSelection("Quit Mission");

	AudioManager::Instance()->setVolume("Mission", 75);
	AudioManager::Instance()->resume("Mission");

	mSelectorHeight = 0;
	mSelectorHeight = mPauseMenu.CurrentSelection() * -0.1 + 0.55;
}

// Only two options: continue game or quit mission.
void PauseState::HandleInput(unsigned char key)
{
	// ESCAPE continues main game
	if(27 == key)
	{
		AudioManager::Instance()->pause("Mission");
		GameFramework::Instance()->SetState(PilotState::Instance());
	}

	// arrow keys change selection
	if(GLUT_KEY_UP == key)
	{
		mPauseMenu.SelectPrevious();
		mSelectorHeight = mPauseMenu.CurrentSelection() * -0.1 + 0.55;
	}
	if(GLUT_KEY_DOWN == key)
	{
		mPauseMenu.SelectNext();
		mSelectorHeight = mPauseMenu.CurrentSelection() * -0.1 + 0.55;
	}

	// ENTER or SPACE selects a menu option
	if((13 == key) || (' ' == key))
	{
		switch(mPauseMenu.CurrentSelection())
		{
		case(0):
			AudioManager::Instance()->pause("Mission");
			GameFramework::Instance()->SetState(PilotState::Instance());
			break;
		case(1):
			AudioManager::Instance()->unload("Mission");
			GameFramework::Instance()->SetState(MenuState::Instance());
			break;
		}
	}
}

// Draw the ship pointer, FPS (if visible), and menu
void PauseState::Render()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// do this for every state that has a spaceship pointer (mainMenu, options, pause)
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0.0, 1.0, 0.0, 1.0);

	glColor3f(1.0, 0.2, 0.0);
	glBegin(GL_LINE_LOOP);
		glVertex3f(0.34, mSelectorHeight + 0.03, 0.0);
		glVertex3f(0.34, mSelectorHeight - 0.03, 0.0);
		glVertex3f(0.4, mSelectorHeight, 0.0);
	glEnd();

	glColor3f(1.0, 1.0, 1.0);
	GameFramework::Instance()->DrawFPS();

	mPauseMenu.Render();
}

// HandleInput takes care of everything
void PauseState::Update(double timeDelta, unsigned char *keyBuffer)
{
}

// Free the loaded music.
void PauseState::CleanUp()
{
	mPauseMenu.CleanUp();
}
