////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// gsStory.cpp
////////////////////////////////////////////////////////////////////////////////

#include "gameStates.h"

////////////////////////////////////////////////////////////////////////////////
// StoryState displays the back story for each level.
////////////////////////////////////////////////////////////////////////////////

// Declare the single instance.
StoryState *StoryState::mInstance = 0;

// Load the text of the story that should be shown before the current mission.
// Also, play the story music.
void StoryState::Initialize()
{
	std::string line;
	std::fstream storyFile;
	char lineAsChars[80];
	std::string fileName = "assets\\text\\story";
	fileName += (48 + GameFramework::Instance()->GetSharedData(MissionLevel));
	fileName.append(".txt");
	mStoryLines.clear();

	glClearColor(0.0, 0.0, 0.0, 1.0);

	// This is a strange way of changing the state to alert the level loader
	// that a new game is beginning and the ship's position and orientation
	// should be reloaded from file.
	AudioManager::Instance()->unload("Mission");

	AudioManager::Instance()->load("The Jedi", "assets\\audio\\jedi.mp3");
	AudioManager::Instance()->setVolume("The Jedi", 100);
	AudioManager::Instance()->play("The Jedi");

	storyFile.open(fileName.c_str(), std::ios::in);

	if(storyFile.is_open())
	{
		while(!storyFile.eof())
		{
			storyFile.getline(lineAsChars, 80);
			line.assign(lineAsChars);
			mStoryLines.push_back(line);
		}

		storyFile.close();
	}
	
	GameFramework::Instance()->SetSharedData(LevelSubState, SUB_STATE_PLAYING);
}

// Any key presse begins the countdown to the mission.
void StoryState::HandleInput(unsigned char key)
{
	if(key)
	{
		if(0 <= Ship::Instance()->getLives() &&
			GAME_BEAT_LEVEL != GameFramework::Instance()->GetSharedData(MissionLevel))
		{
			GameFramework::Instance()->SetState(CountState::Instance());
		}
		if(0 > Ship::Instance()->getLives())
		{
			// game over
			GameFramework::Instance()->SetState(MenuState::Instance());
		}
		if(GAME_BEAT_LEVEL == GameFramework::Instance()->GetSharedData(MissionLevel))
		{
			// won game
			GameFramework::Instance()->SetState(CreditsState::Instance());
		}
	}
}

// Draw the countdown depending on what events have happened.
void StoryState::Render()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glColor3f(1.0, 1.0, 1.0);
	GameFramework::Instance()->DrawFPS();

	for(int i = 0; i < mStoryLines.size(); i++)
	{
		GameFramework::Instance()->DrawText(mStoryLines[i], 0.0, (-0.08 * i) + 0.9, true);
	}
}

// nothing changes here that is not covered in HandleInput()
void StoryState::Update(double timeDelta, unsigned char *keyBuffer)
{
}

// Free loaded music.
void StoryState::CleanUp()
{
	AudioManager::Instance()->unload("The Jedi");
}
