////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// gsTitle.cpp
////////////////////////////////////////////////////////////////////////////////

#include "gameStates.h"

////////////////////////////////////////////////////////////////////////////////
// TitleState displays the title screen with the launching spaceship.
////////////////////////////////////////////////////////////////////////////////

// Declare the single instance.
TitleState *TitleState::mInstance = 0;

// Load level from file.
void TitleState::Initialize()
{
	// enable alpha blending for "in" text fade in
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	fadeText = 0.0;

	// play the THX theme
	AudioManager::Instance()->load("THX", "assets\\audio\\thx.mp3");
	AudioManager::Instance()->setVolume("THX", 100);
	AudioManager::Instance()->play("THX");

	// The title screen is stored in a format similar to the missions.
	Level::Instance()->LoadFromFile("assets\\text\\title.txt");

	// allocate memory for stars, remember to delete
	mStars = new Vector3d[STAR_COUNT_TITLE];

	// position stars so they appear above the horizon
	for(int i = 0; i < STAR_COUNT_TITLE; ++i)
	{
		mStars[i].setX((rand() % GAME_SCREEN_WIDTH/2));
		mStars[i].setY((rand() % GAME_SCREEN_HEIGHT/2));
		mStars[i].setZ(0.0);
	}

	// initial view is 2 units wide by 1.5 units tall and centered in the ship
	// which is making up part of the capital 'A' in "Ace Lander"
	mLowerLeft = Vector3d(38.2, 52.0, 0.0); // 19, 24
	mUpperRight = Vector3d(39.2, 52.75, 0.0); // 21, 25.5

	mEndTitle.SetDelay(17.76);
	mEndZoom.SetDelay(ZOOM_TIME);
	mIgnition.SetDelay(8.0);
	mFadeText.SetDelay(13.0);

	Ship::Instance()->setVelocity(Vector3d(0.0, 8.0, 0.0));
}

void TitleState::HandleInput(unsigned char key)
{
	if(key)
	{
		GameFramework::Instance()->SetState(IntroState::Instance());
	}
}

// Draw the ship and title level.
void TitleState::Render()
{
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// draw stars in the sky
	glColor3f(1.0, 1.0, 1.0);
	glBegin(GL_POINTS);
		for(int i = 0; i < STAR_COUNT_TITLE; i++)
		{
			glVertex3d(mStars[i].getX(), mStars[i].getY(), 0.0);
		}
	glEnd();

	// draw the ship and title
	Level::Instance()->Render();
	Ship::Instance()->Render(2.0);

	// draw fading in "in"
	glColor4f(1.0, 1.0, 1.0, fadeText);
	GameFramework::Instance()->DrawText("in", 0.0, 0.12, true);

	// draw FPS
	glColor4f(1.0, 1.0, 1.0, 1.0);
	GameFramework::Instance()->DrawFPS();
}

// zoom out from the ship in 'A' to view the background of stars and launch ship
void TitleState::Update(double timeDelta, unsigned char *keyBuffer)
{
	// fade in "in" near end of title zoom
	if(mFadeText.HasSignaled())
	{
		fadeText += 0.5 * timeDelta;
		if(fadeText > 1.0)
		{
			fadeText = 1.0;
		}
	}

	// desired coordinates of final window: (-57, -80), (343, 220)
	if(!mEndZoom.HasSignaled())
	{
		mLowerLeft.setX(mLowerLeft.getX() - (9.5 - -28.5) / ZOOM_TIME * timeDelta);
		mLowerLeft.setY(mLowerLeft.getY() - (12.0 - -40.0) / ZOOM_TIME * timeDelta);
		mUpperRight.setX(mUpperRight.getX() + (171.5 - 10.5) / ZOOM_TIME * timeDelta);
		mUpperRight.setY(mUpperRight.getY() + (110.0 - 12.75) / ZOOM_TIME * timeDelta);
	}

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(mLowerLeft.getX(), mUpperRight.getX(), mLowerLeft.getY(), mUpperRight.getY());

	// move ship up (at a constant rate, not physically accelerating)
	if(mIgnition.HasSignaled())
	{
		Vector3d position = Ship::Instance()->getPosition();
		Ship::Instance()->igniteEngines();
		Ship::Instance()->setPosition(Vector3d(position.getX(),
			position.getY() + SHIP_CONSTANT_SPEED * timeDelta, 0.0));
		Ship::Instance()->SimulateParticles(timeDelta);
	}

	// change state to intro cinematic
	if(mEndTitle.HasSignaled())
	{
		GameFramework::Instance()->SetState(IntroState::Instance());
	}
	
	mEndTitle.Update(timeDelta);
	mEndZoom.Update(timeDelta);
	mIgnition.Update(timeDelta);
	mFadeText.Update(timeDelta);
}

// free loaded music and star array
void TitleState::CleanUp()
{
	AudioManager::Instance()->unload("THX");
	delete [] mStars;
}
