////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// level.h
////////////////////////////////////////////////////////////////////////////////

#ifndef LEVEL_H
#define LEVEL_H

#include "ship.h"
#include "gameStates.h"
#include "framework.h"

// gameplay screen size
const int GAME_SCREEN_WIDTH = 400;
const int GAME_SCREEN_HEIGHT = 300;
// landing platform size
const double PLATFORM_WIDTH = 10.0;
const double PLATFORM_HEIGHT = 2.5;
// ship center can be this far from edges of platform before collision is considered a crash
const double HORIZONTAL_VARIANCE = 0.7;
// orientation of ship is allowed to vary this much before collision is considered a crash
const double ROTATIONAL_VARIANCE = 0.3;
// maximum speed ship can travel when landing before collision is considered a crash
const double MAXIMUM_LANDING_SPEED = 6.6;

////////////////////////////////////////////////////////////////////////////////
// A Level consists of terrain made up of line segments, a landing pad,
// and a ship.  Levels can be loaded from file.
////////////////////////////////////////////////////////////////////////////////

// the maximum number of concave polygons supported by a level
const int MAX_POLY_COUNT = 25;

class Level
{
public:
	~Level() { mInstance = NULL; };
	// Returns the only instance of the class, creating it if it doesn't exist yet.
	static Level *Instance() { if(0==mInstance){mInstance=new Level;}return mInstance; };
	void LoadFromFile(std::string fileName);
	void CheckCollisions();
	void ZoomCamera();
	void Render();
	void RenderOverlayStats();

protected:
	Level() { };

private:
	static Level *mInstance;

	std::vector<Vector3d> mTerrain;
	std::vector<int> mPolyBreaks;
	Vector3d mPlatform[4];
	int mConcavePolyCount;
	int mPointsPerPoly[MAX_POLY_COUNT];
};

#endif
