////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// ship.h
////////////////////////////////////////////////////////////////////////////////

#ifndef SHIP_H
#define SHIP_H

#include "framework.h"
#include "particle.h"

// rotation rate of ship (radians per second)
const double ROTATION_RATE = 1.3;
// rate at which fuel is used up (units per second)
const double FUEL_CONSUMPTION_RATE = 0.0495;
// magnitude of the vector describing thrust
const double THRUST_MAGNITUDE = 29.0;
// from center of ship to bow vertex
const Vector3d BOW(0.0, 2.0, 0.0);
// from center of ship to port vertex
const Vector3d PORT(-1.0, -1.0, 0.0);
// from center of ship to starboard vertex
const Vector3d STARBOARD(1.0, -1.0, 0.0);

////////////////////////////////////////////////////////////////////////////////
// The Ship is used in the title screen, the introduction cinematic,
// as a selection indicator in the menus, and the main game.
////////////////////////////////////////////////////////////////////////////////
class Ship
{
public:
	~Ship() { mInstance = NULL; };
	// Returns the only instance of the class, creating it if it doesn't exist yet.
	static Ship *Instance() { if(0==mInstance){mInstance=new Ship;}return mInstance; };
	void Initialize();
	void Simulate(double timeDelta, unsigned char *keyBuffer);
	void Render(double scale = 1.0, double z = 0.0);

	// cinematic mutators (these are used for intro and menu manipulation of ship)
	void igniteEngines();
	void SimulateParticles(double timeDelta);
	void setPosition(Vector3d position);
	void setOrientation(double newOrientation);
	// used to reinitialize ship when level is started
	void setVelocity(Vector3d velocity);
	void setFuel(double fuel);

	// accessors needed to check collisions within level
	const Vector3d &getBow() const;
	const Vector3d &getPort() const;
	const Vector3d &getStarboard() const;
	const Vector3d &getPosition() const;
	const Vector3d &getVelocity() const;
	double getOrientation() const;

	// used by the pilot state to draw the fuel meter and life stock and update the number of ships
	double getFuel() const;
	int getLives() const;
	void crash();

protected:
	Ship() { };

private:
	static Ship *mInstance;

	// Vectors involved in simulating position and motion of ship.
	Vector3d mPosition;
	Vector3d mVelocity;
	Vector3d mGravity;
	Vector3d mThrust;

	// Vectors describing outline of ship relative to its center (position vector).
	Vector3d mBow;
	Vector3d mPort;
	Vector3d mStarboard;

	double mOrientation;			// angle in radians from vertical
	double mFuel;					// amount of fuel remaining (max 1.0 unit)
	int mLives;						// chances to continue after a crash
	Exhaust mExhaust;				// System describing the exhaust of the engines
};

#endif
